/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.pkgview;

import java.util.List;
import java.util.ArrayList;
import javax.swing.table.AbstractTableModel;

/**
 * A TableModel describing Solaris packages.
 * @author Peter Tribble
 * @version 1.0
 */
public class PackageTableModel extends AbstractTableModel {

    private boolean showselected;

    private List <SolarisPackage> pkgs;

    /*
     * Columns to show
     */
    private String[] col1 = { "Name", "Category", "Description" };
    private String[] col2 = { "Selected", "Name", "Category", "Description" };
    private String[] columnNames;

    /**
     * Create a new PackageTableModel.
     *
     * @param plist a PkgList
     * @param showselected true if the extra first column to show whether
     * packages are installed or selected is to be shown
     */
    public PackageTableModel(PkgList plist, boolean showselected) {
	this.showselected = showselected;
	pkgs = new ArrayList <SolarisPackage> (plist.getPackages());
	columnNames = (showselected) ? col2 : col1;
    }

    public int getColumnCount() {
	return columnNames.length;
    }

    public int getRowCount() {
	return pkgs.size();
    }

    @Override
    public String getColumnName(int col) {
	return columnNames[col];
    }

    /**
     * Return the appropriate data.
     *
     * @see #setValueAt
     *
     * @param row the int row of the selected cell
     * @param col the int column of the selected cell
     *
     * @return the Object at the selected cell
     */
    public Object getValueAt(int row, int col) {
	SolarisPackage pkg = pkgs.get(row);
	if (showselected) {
	    if (col == 0) {
		return Boolean.valueOf(pkg.isSelected());
	    } else {
		col--;
	    }
	}
	if (col == 0) {
	    return pkg.getName();
	} else if (col == 1) {
	    String s =  pkg.getInfoItem("CATEGORY");
	    return (s == null) ? "" : s.split(",")[0];
	} else {
	    return pkg.getDescription();
	}
    }

    @Override
    public Class<?> getColumnClass(int col) {
	return (showselected && (col == 0)) ? Boolean.class : String.class;
    }

    /*
     * The only editable cells are those in the first column and only then
     * if we're showing the selection. Furthermore, if the package cannot
     * be unselected then it shouldn't be editable ether.
     *
     * FIXME re-enable; it's been disabled because there's nothing that
     * connects the selection to the profile
     */
    /*
     * @Override
     * public boolean isCellEditable(int row, int col) {
     *	if (showselected && (col == 0)) {
     *	    return !getPackageAtRow(row).isRequired();
     *	}
     *	return false;
     * }
     */

    /*
     * Don't need to implement this method unless your table's
     * data can change.
     */
    @Override
    public void setValueAt(Object value, int row, int col) {
	if (showselected && (col == 0)) {
	    Boolean b = (Boolean) value;
	    getPackageAtRow(row).setSelected(b.booleanValue());
	    fireTableCellUpdated(row, col);
	}
    }

    /**
     * Return the package at the selected row.
     *
     * @param row the row to select
     *
     * @return the SolarisPackage on the given row
     */
    public SolarisPackage getPackageAtRow(int row) {
	return pkgs.get(row);
    }
}
